@@ -3,7 +3,7 @@
 /*
  *  Quick Lounge Applet
  *
- *  Copyright (C) 2001 The Free Software Foundation, Inc.
+ *  Copyright (C) 2001, 2004 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 #include <string.h>
 #include <math.h>
 #include <gtk/gtk.h>
+#include <gconf/gconf-client.h>
 #include <libgnome/gnome-desktop-item.h>
 #include <libgnomeui/gnome-ditem-edit.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
@@ -34,6 +35,7 @@
 #include "quick-button.h"
 #include "quick-separator.h"
 #include "gtk-utils.h"
+#include "xstuff.h"
 
 #define ICON_SPACING         1       /* spacing between icons */
 #define ICON_SIZE_PANEL      24      /* icons size on panels */
@@ -186,7 +188,7 @@ quick_box_size_request (GtkWidget        *widget,
 	min_columns = MAX (min_columns, 0);
 
 	max_columns = p->max_visible_cols;
-	max_columns = MAX (max_columns, 1);
+	max_columns = MAX (max_columns, 0);
 	max_columns = MAX (max_columns, min_columns);
 
 	/**/
@@ -256,28 +258,28 @@ quick_box_size_request (GtkWidget        *widget,
 			   && QUICK_IS_SEPARATOR (child->widget)) {
 			
 			if (max_column_height != 0) {
-				p->col_size[col - 1] = max_column_height + spacing;
-				p->buttons_on_col[col - 1] = row - 1;
-				cur_size += p->col_size[col - 1];
-				if (col == min_columns)
+				p->col_size[row - 1] = max_column_height + spacing;
+				p->buttons_on_col[row - 1] = col - 1;
+				cur_size += p->col_size[row - 1];
+				if (row == min_columns)
 					min_size = cur_size;
-				if (col == max_columns)
+				if (row == max_columns)
 					max_size = cur_size;
 				max_column_height = 0;
-				col++;
-				row = 1;
+				row++;
+				col = 1;
 			}
 			
-			p->col_size[col - 1] = child_requisition.height + spacing;
-			p->buttons_on_col[col - 1] = 1;
-			cur_size += p->col_size[col - 1];
-			if (col == min_columns)
+			p->col_size[row - 1] = child_requisition.height + spacing;
+			p->buttons_on_col[row - 1] = 1;
+			cur_size += p->col_size[row - 1];
+			if (row == min_columns)
 				min_size = cur_size;
-			if (col == max_columns)
+			if (row == max_columns)
 				max_size = cur_size;
 			max_column_height = 0;
-			col++;
-			row = 1;
+			row++;
+			col = 1;
 
 		} else if ((orientation == GTK_ORIENTATION_VERTICAL)
 			   && QUICK_IS_BUTTON (child->widget)) {
@@ -285,17 +287,17 @@ quick_box_size_request (GtkWidget        *widget,
 			max_column_height = MAX (max_column_height, 
 						 child_requisition.height);
 
-			if (row++ == p->rows) {
-				p->col_size[col - 1] = max_column_height + spacing;
-				p->buttons_on_col[col - 1] = row - 1;
-				cur_size += p->col_size[col - 1];
-				if (col == min_columns)
+			if (col++ == p->rows) {
+				p->col_size[row - 1] = max_column_height + spacing;
+				p->buttons_on_col[row - 1] = col - 1;
+				cur_size += p->col_size[row - 1];
+				if (row == min_columns)
 					min_size = cur_size;
-				if (col == max_columns)
+				if (row == max_columns)
 					max_size = cur_size;
 				max_column_height = 0;
-				col++;
-				row = 1;
+				row++;
+				col = 1;
 			}
 		}
 	}
@@ -308,16 +310,23 @@ quick_box_size_request (GtkWidget        *widget,
 		}
 	} else {
 		if (max_column_height != 0) {
-			p->col_size[col - 1] = max_column_height + spacing;
-			p->buttons_on_col[col - 1] = row - 1;
-			cur_size += p->col_size[col - 1];
+			p->col_size[row - 1] = max_column_height + spacing;
+			p->buttons_on_col[row - 1] = col - 1;
+			cur_size += p->col_size[row - 1];
 		}
 	}
 
-	if (col <= min_columns)
-		min_size = cur_size;
-	if (col <= max_columns)
-		max_size = cur_size;
+	if (orientation == GTK_ORIENTATION_HORIZONTAL) {
+		if (col <= min_columns)
+			min_size = cur_size;
+		if (col <= max_columns)
+			max_size = cur_size;
+	} else {
+		if (row <= min_columns)
+			min_size = cur_size;
+		if (row <= max_columns)
+			max_size = cur_size;
+	}
 
 	min_size += spacing + INITIAL_OFFSET * 2;
 	max_size += spacing + INITIAL_OFFSET * 2;
@@ -332,30 +341,21 @@ quick_box_size_request (GtkWidget        *widget,
 	if (orientation == GTK_ORIENTATION_HORIZONTAL) { 
 		min_size += spacing + arrow_width;
 		max_size += spacing + arrow_width;
+		if (box->children == NULL) /* when the box is empty */
+			min_size = p->size / p->rows; 
+		requisition->width = min_size;
+		requisition->height = MAX (p->size, arrow_height);
+		p->size_hint[SIZE_HINT_MAX] = max_size - requisition->width;
 	} else {
 		min_size += spacing + arrow_height;
 		max_size += spacing + arrow_height;
-	}
-
-	if (box->children == NULL) /* when the box is empty */
-		min_size = p->size / p->rows; 
-
-	/**/
-
-	if (orientation == GTK_ORIENTATION_HORIZONTAL) {
-		requisition->width = min_size;
-		requisition->height = MAX (p->size, arrow_height);
-	} else {
+		if (box->children == NULL) /* when the box is empty */
+			min_size = p->size / p->rows; 
 		requisition->width = MAX (p->size, arrow_width);
 		requisition->height = min_size;
+		p->size_hint[SIZE_HINT_MAX] = max_size - requisition->height;
 	}
 
-	/**/
-
-	if (orientation == GTK_ORIENTATION_HORIZONTAL) 
-		p->size_hint[SIZE_HINT_MAX] = max_size - requisition->width;
-	else
-		p->size_hint[SIZE_HINT_MAX] = max_size - requisition->height;
 	p->size_hint[SIZE_HINT_MIN] = 0;
 
 	/* FIXME
@@ -371,6 +371,10 @@ quick_box_size_request (GtkWidget        *widget,
 
 	requisition->width += GTK_CONTAINER (box)->border_width * 2;
 	requisition->height += GTK_CONTAINER (box)->border_width * 2;
+
+#ifdef DEBUG
+        g_print ("REQ [%d, %d]", requisition->width, requisition->height);
+#endif
 }
 
 
@@ -563,11 +567,9 @@ quick_box_size_allocate (GtkWidget        *widget,
 		initial_y = (allocation->height - (get_child_size (quick_box) + ICON_SPACING) * quick_box->priv->rows) / 2;
 
 	} else {
-		initial_x = allocation->x + border_width;
-
 		/* center horizontally */
 		initial_x = (allocation->width - (get_child_size (quick_box) + ICON_SPACING) * quick_box->priv->rows) / 2;
-		
+
 		initial_y = allocation->y + border_width + spacing + INITIAL_OFFSET;
 	}
 
@@ -587,7 +589,10 @@ quick_box_size_allocate (GtkWidget        *widget,
 	current_width  = initial_x;
 	current_height = initial_y;
 
-	under_arrow_children = children_under_the_arrow (quick_box, arrow_width);
+	if (orientation == GTK_ORIENTATION_HORIZONTAL)
+		under_arrow_children = children_under_the_arrow (quick_box, arrow_width);
+	else
+		under_arrow_children = children_under_the_arrow (quick_box, arrow_height);
 	ignoring_arrow = TRUE;
 
 	for (children = box->children; children; ) {
@@ -614,13 +619,15 @@ quick_box_size_allocate (GtkWidget        *widget,
 		else
 			current_width += child_allocation.width;
 
-#ifdef DEBUG
-		g_print ("\n");
-		g_print ("[X] %d <--> %d\n", x + child_allocation.width, max_width);
-		g_print ("[Y] %d <--> %d\n", current_height, max_height);
-#endif
 
 		if (orientation == GTK_ORIENTATION_HORIZONTAL) {
+
+#ifdef DEBUG
+			g_print ("\n");
+			g_print ("[X] %d <--> %d\n", x + child_allocation.width, max_width);
+			g_print ("[Y] %d <--> %d\n", current_height, max_height);
+#endif
+			
 			if (x + child_allocation.width > max_width) {
 				if (! do_not_fit) {
 					do_not_fit = TRUE;
@@ -632,14 +639,22 @@ quick_box_size_allocate (GtkWidget        *widget,
 					hide_from_here = children;
 				} else {
 					x += max_line_width + spacing;
-					y  = initial_y;
-					current_height  = border_width;
-					max_line_width  = 0;
+					y = initial_y;
+					current_height = border_width;
+					max_line_width = 0;
 					second_try = TRUE;
 					continue;
 				}
 			}
 		} else {
+			
+#ifdef DEBUG
+			g_print ("\n");
+			g_print ("[X] %d <--> %d\n", current_width, max_width);
+			g_print ("[Y] %d <--> %d\n", y + child_allocation.height, max_height);
+			
+#endif
+			
 			if (y + child_allocation.height > max_height) {
 				if (! do_not_fit) {
 					do_not_fit = TRUE;
@@ -650,10 +665,10 @@ quick_box_size_allocate (GtkWidget        *widget,
 					do_not_fit = TRUE;
 					hide_from_here = children;
 				} else {
-					x  = initial_x;
+					x = initial_x;
 					y += max_line_height + spacing;
-					current_width    = border_width;
-					max_line_height  = 0;
+					current_width = border_width;
+					max_line_height = 0;
 					second_try = TRUE;
 					continue;
 				}
@@ -890,6 +905,19 @@ quick_box_class_init (QuickBoxClass    *class)
 }
 
 
+static gboolean
+enable_animations (void)
+{
+	GConfClient *client = gconf_client_get_default ();
+	gboolean     global_animation, panel_animation;
+
+	global_animation = gconf_client_get_bool (client, "/desktop/gnome/interface/enable_animations", NULL);
+	panel_animation = gconf_client_get_bool (client, "/apps/panel/global/enable_animations", NULL);
+
+	return global_animation && panel_animation;
+}
+
+
 static gboolean
 item_activated (QuickButton *button)
 {
@@ -899,6 +927,9 @@ item_activated (QuickButton *button)
 	if (ditem != NULL) {
 		GError  *error = NULL;
 
+		if (enable_animations ())
+			xstuff_zoom_animate (GTK_WIDGET (button), NULL);
+
 		gnome_desktop_item_launch_on_screen (ditem, 
 						     NULL,
 						     0,
@@ -1065,6 +1096,11 @@ arrow_button_pressed (GtkToggleButton *tbutton,
 	GList      *children;
 	gboolean    menu_empty = TRUE;
 
+	gtk_tooltips_set_tip (quick_box->tooltips, 
+			      GTK_WIDGET (tbutton),
+			      _("Hide launchers menu"),
+			      NULL);
+
 	if (quick_box->priv->popup_menu != NULL) {
 		gtk_menu_shell_deactivate (GTK_MENU_SHELL (quick_box->priv->popup_menu));
 		return TRUE;
@@ -1207,6 +1243,10 @@ deactivate_arrow (QuickBox *quick_box)
 					 quick_box);
 	
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (arrow), FALSE);
+	gtk_tooltips_set_tip (quick_box->tooltips, 
+			      arrow,
+			      _("Show hidden launchers"),
+			      NULL);
 	
 	g_signal_handlers_unblock_by_func (G_OBJECT (arrow),
 					   G_CALLBACK (arrow_button_pressed),
@@ -1231,6 +1271,11 @@ create_arrow_button (QuickBox     *quick_box,
 	gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
 	gtk_widget_set_parent (button, GTK_WIDGET (quick_box));
 
+	gtk_tooltips_set_tip (quick_box->tooltips, 
+			      button,
+			      _("Show hidden launchers"),
+			      NULL);
+
 	g_signal_connect (G_OBJECT (button), 
 			  "toggled",
 			  G_CALLBACK (arrow_button_pressed), 
@@ -1265,14 +1310,14 @@ quick_box_init (QuickBox *quick_box)
 	priv->populate_menu_func = NULL;
 	priv->populate_menu_data = NULL;
 
+	quick_box->tooltips = gtk_tooltips_new ();
+	g_object_ref (quick_box->tooltips);
+	gtk_object_sink (GTK_OBJECT (quick_box->tooltips));
+
 	priv->arrow_up    = create_arrow_button (quick_box, GTK_ARROW_UP);
 	priv->arrow_down  = create_arrow_button (quick_box, GTK_ARROW_DOWN);
 	priv->arrow_left  = create_arrow_button (quick_box, GTK_ARROW_LEFT);
 	priv->arrow_right = create_arrow_button (quick_box, GTK_ARROW_RIGHT);
-
-	quick_box->tooltips = gtk_tooltips_new ();
-	g_object_ref (quick_box->tooltips);
-	gtk_object_sink (GTK_OBJECT (quick_box->tooltips));
 }
 
 
@@ -1481,7 +1526,10 @@ quick_box_add_button (QuickBox     *quick_box,
 	/* tip */
 
 	comment = gnome_desktop_item_get_localestring (ditem, "Comment"); 
-	tip = g_strconcat (((comment != NULL) && (*comment != 0)) ? comment : text, NULL);
+	tip = g_strconcat (text,
+			   ((comment != NULL) && (*comment != 0)) ? "\n" : NULL,
+			   ((comment != NULL) && (*comment != 0)) ? comment: NULL,
+			   NULL);
 	gtk_tooltips_set_tip (quick_box->tooltips, 
 			      button,
 			      tip,
